// Déclaration des bibliothèques utilisées
#include <lcdgfx.h>
#include <WiFiClientSecure.h>

// Inclusion des fichiers du projet
#include "image.h"

// Définition des constantes globales
#define SSID_L          "LiveBox_1324" // Nom du réseau Wi-Fi
#define MOT_DE_PASSE_L  "12345?ABCDE" // Mot De Passe du réseau Wi-Fi
  
// Définition des types
typedef struct
{
  char id[20];
  PGM_VOID_P image;
} crypto_t;

// Prototypes de fonction
void GestionConnexionWifi(void);
void DemandeCoinGecko(char*, char*);
void ConversionMinuscule(char*, char*);
bool ExtractionCours(char*, char*);
void AfficherCrypto(char*, crypto_t);

const PROGMEM uint8_t ssd1306_fonte_8x16_CaracteresSpeciaux []=
{
  0x01, // 
  0x08, // 0x08 = 8 - largeur de la police en pixels
  0x10, // 0x10 = 16 - hauteur de la police en pixels
  0x20,
  0x00, 0x82, 0x01, // Première partie du caractère Euro
  0x00, 0x00, 0x00, 0x80, 0x80, 0xE0, 0xB0, 0x98, 0x00, 0x00, 0x00, 0x02, 0x02, 0x0F, 0x1A, 0x12, // Symbole de l'euro
  0x00, 0xAC, 0x01, // Deuxième partie du caractère Euro
  0x88, 0x88, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x32, 0x22, 0x20, 0x00, 0x00, 0x00, 0x00, 0x00, // Symbole de l'euro
  0x00, 0x00, 0x00, // Fin du tableau de caractère
};

// Déclarations des variables globales
WiFiClientSecure ClientWeb_G;
DisplaySSD1306_128x64_I2C Afficheur_G(16,{-1, 0, 15, 4, 0});

crypto_t cryptos[] =
{
  {"Bitcoin", ImageBitcoin}
};

// Fonction de démarrage, s'exécute une seule fois:
void setup()
{
  /* Initialisation de l'écran */
  Afficheur_G.begin();
  /* Effacement de l'écran */
  Afficheur_G.clear();
  /* Sélection des fontes de caractères */
  Afficheur_G.setFixedFont(ssd1306xled_font8x16, ssd1306_fonte_8x16_CaracteresSpeciaux);

  Serial.begin(115200); // Ouverture du port série à 115200 bauds
  
  GestionConnexionWifi();
}

// Fonction principale du programme, s'exécute en boucle:
void loop()
{
  // Variables de la fonction
   char cours_l[20], reponse_coingecko[150];

  GestionConnexionWifi();
  DemandeCoinGecko(reponse_coingecko, "Bitcoin");
  if(ExtractionCours(cours_l, reponse_coingecko))
  {
    AfficherCrypto(cours_l, cryptos[0]);
  }
  else
  {
    Serial.print("Impossible d'obtenir la valeur du Bitcoin");
  }
  ClientWeb_G.stop();
  delay(10000); // Attente de 10 secondes
}

void GestionConnexionWifi()
{
  // Variables de la fonction
  static wl_status_t StatutConnexion_L = WL_DISCONNECTED; // Pour mémoriser l'état de la connexion

  // Verification du statut de la connexion
  StatutConnexion_L = WiFi.status();

  if(WiFi.status()!=WL_CONNECTED)
  {
    Serial.println("Connexion au point d'accès Wi-Fi ...");
    WiFi.begin(SSID_L, MOT_DE_PASSE_L); // Tentative de connexion au point d'accès Wi-Fi
    StatutConnexion_L = WiFi.status(); // Lecture de l'état de la connexion et mémorisation dans la variable "StatutConnexion_L"
    while ((StatutConnexion_L != WL_NO_SSID_AVAIL)&&(StatutConnexion_L != WL_CONNECTED)&&(StatutConnexion_L != WL_CONNECT_FAILED))
    {
      Afficheur_G.drawBitmap1(95, 0, 32, 32, ImageWifiNiveau_0);
      delay(300);
      Afficheur_G.drawBitmap1(95, 0, 32, 32, ImageWifiNiveau_1);
      delay(300);
      Afficheur_G.drawBitmap1(95, 0, 32, 32, ImageWifiNiveau_2);
      delay(300);
      Afficheur_G.drawBitmap1(95, 0, 32, 32, ImageWifiNiveau_3);
      delay(300);
      StatutConnexion_L = WiFi.status(); // Lecture de l'état de la connexion et mémorisation dans la variable "StatutConnexion_L"
    }
    // Affichage du résultat de la tentative de connexion
    if (StatutConnexion_L == WL_CONNECTED)
    {
      Serial.println("Connection OK");
      ClientWeb_G.setInsecure();  //  On ne vérifie pas les certificats ni les empreintes https 
    }
    else if (StatutConnexion_L == WL_NO_SSID_AVAIL)
    {
      Afficheur_G.drawBitmap1( 95, 0, 32, 32, ImageWifiNiveau_0);
      Serial.println("SSID introuvable");
    }
    else if (StatutConnexion_L == WL_CONNECT_FAILED)
    {
      Afficheur_G.drawBitmap1( 95, 0, 32, 32, ImageWifiNiveau_0);
      Serial.println("Mot de passe KO");
    }
    else
    {
      Afficheur_G.drawBitmap1( 95, 0, 32, 32, ImageWifiNiveau_0);
      Serial.println("Autre erreur");
    }   
  }  
}

void DemandeCoinGecko(char *reponse_p, char *crypto_id_p)
{
  char requete_l[150], id_minuscule_l[20];
  int index_donnees_l;
  
  while(!ClientWeb_G.connected())
  {
    if(WiFi.status()==WL_CONNECTED)
    {    
      ClientWeb_G.connect("api.coingecko.com", 443);
      Serial.println("Connexion à CoinGecko ...");
    }
    else
    {
      GestionConnexionWifi();
    }
  }
  ConversionMinuscule(id_minuscule_l, crypto_id_p);
  sprintf(requete_l, "GET https://api.coingecko.com/api/v3/simple/price?ids=%s&vs_currencies=eur HTTP/1.0", id_minuscule_l);
  Serial.print("requete_l=");
  Serial.println(requete_l);
  ClientWeb_G.println(requete_l);
  ClientWeb_G.println("Host: api.coingecko.com");
  ClientWeb_G.println("Connection: close");
  ClientWeb_G.println();
  
  while(ClientWeb_G.connected() && !ClientWeb_G.available())
  {
    delay(1); // On attend la réponse du serveur
  }
  while (ClientWeb_G.available()&&(ClientWeb_G.readStringUntil('\n')!="\r")); // On évacue l'entête de la réponse
  for(index_donnees_l=0; ClientWeb_G.available(); index_donnees_l++)
  {
    reponse_p[index_donnees_l] = (char)ClientWeb_G.read(); 
  }
  ClientWeb_G.stop();
  reponse_p[index_donnees_l] = NULL;  // Ajout du caractère de fin de chaine
  Serial.print("reponse_p=");
  Serial.println(reponse_p);
}

void ConversionMinuscule(char *destination_p, char *source_p)
{
  for(int i = 0; source_p[i]; i++)
  {
    destination_p[i] = tolower(source_p[i]);
  }
  destination_p[strlen(source_p)] = NULL; // Ajout du caractère de fin de chaîne
}

bool ExtractionCours(char *valeur_p, char *reponse_p)
{
  bool resultat_l = false;
  char partie_entiere[10], partie_decimale[10]; 
  const char* index_2points_l = strrchr(reponse_p, ':');
  const char* index_accolade_fermante_l = strchr(reponse_p, '}');
  int longueur_valeur_l;
  char* index_point_l;
  
  if ((index_2points_l != NULL)&&(index_accolade_fermante_l != NULL))
  {
    longueur_valeur_l=index_accolade_fermante_l-(index_2points_l+1);
    strncpy(valeur_p, index_2points_l+1, longueur_valeur_l);
    valeur_p[longueur_valeur_l] = NULL; // Ajout du caractère de fin de chaîne
    strcat(valeur_p, "€");
    index_point_l =  strchr(valeur_p, '.');
    if (index_point_l != NULL)
    {
      *index_point_l=',';  
    }
    resultat_l = true;  
  }
  return(resultat_l);
}

void AfficherCrypto(char *cours_p, crypto_t crypto_p)
{
  Afficheur_G.setColor(0);
  Afficheur_G.fillRect(0, 45, 127, 63);
  Afficheur_G.setColor(0xFFFF);
  Afficheur_G.printFixed (0, 8, crypto_p.id);
  Afficheur_G.drawBitmap1(0, 29, 32, 32, (const uint8_t*)crypto_p.image);
  Afficheur_G.printFixed (127-(8*(strlen(cours_p))), 45, cours_p); // On ajoute le +1 car le signe euros s'étale sur 2 caractères
}
